<?php

namespace Drupal\graphql_fragment_include\GraphQL\Fragment;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * An implementation of GraphqlFragmentLoaderInterface.
 *
 * Allows inclusion of GraphQL Fragments on a query, defined by the
 * following syntax:
 * # include Fragment.gql
 * Includes are not supported by GraphQL, so we need to use comments for them.
 */
class GraphQLFragmentLoader implements GraphQLFragmentLoaderInterface {

  /**
   * Logger instance for this module.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Config object for this module.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Loaded includes.
   *
   * @var array
   */
  protected $loadedFragments;

  /**
   * Constructor for IncludeLoader.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   Logger Factory.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config service.
   */
  public function __construct(
    LoggerChannelFactoryInterface $loggerFactory,
    ConfigFactoryInterface $configFactory
  ) {
    $this->logger = $loggerFactory->get('graphql_fragment_include');
    $this->config = $configFactory->get('graphql_fragment_include.settings');
  }

  /**
   * Find any line defining a fragment include.
   *
   * Returns an array  obtained from executing a regexp. This method should be
   * overridden if include format needs to be changed.
   *
   * @param string $string
   *   The GraphQL string to process.
   *
   * @return array
   *   The matches array obtained from executing a regexp.
   */
  public function findFragments(string $string) {
    preg_match_all("/^#\s+include\s+(\S+)\s*$/m", $string, $matches);
    return $matches;
  }

  /**
   * {@inheritdoc}
   */
  public function loadFragments(string $string) {
    $matches = $this->findFragments($string);
    if ($matches) {
      $baseDir = DRUPAL_ROOT . $this->config->get("fragments_base_dir");
      foreach ($matches[1] as $key => $match) {
        $fragmentFile = $baseDir . '/' . $match;
        if (!is_file($fragmentFile)) {
          $this->logger->warning(t("No fragment '%fragment-file' found.", ['fragment-file' => $fragmentFile]));
          continue;
        }

        // Avoid double loading a fragment.
        if ($this->isFragmentAlreadyLoaded($fragmentFile)) {
          $string = trim(str_replace($matches[0][$key], "", $string));
        }
        else {
          $this->markFragmentAsLoaded($fragmentFile);
          $fragmentData = trim(file_get_contents($fragmentFile));
          // Recursively load other fragments.
          $fragmentData = $this->loadFragments($fragmentData);
          $string = trim(str_replace($matches[0][$key], "", $string));
          $string .= "\n\n# START $match \n" . $fragmentData . "\n# END $match ";
        }
      }
    }
    return $string;
  }

  /**
   * Tells whether a fragment has been already loaded.
   *
   * GraphQL fragments can not be repeated.
   *
   * @param string $fragmentFile
   *   The file to check.
   *
   * @return bool
   *   True if it has been already loaded, false otherwise.
   */
  protected function isFragmentAlreadyLoaded(string $fragmentFile) {
    return isset($this->loadedFragments[$fragmentFile]);
  }

  /**
   * Marks a fragment as already loaded.
   *
   * @param string $fragmentFile
   *   The file to be marked as already loaded.
   */
  protected function markFragmentAsLoaded(string $fragmentFile) {
    $this->loadedFragments[$fragmentFile] = TRUE;
  }

}
